// interface part

#ifdef __cplusplus
extern "C" {
#endif

int init();
int get(int i, int j);
void put(int i, int v);
void done();

#ifdef __cplusplus
}
#endif

// implementation part
// should be in a library, but this is not supported by the grading server

#include <stdio.h>
#include <stdlib.h>

#define __XORLIB_MAX 32767

struct __XORLIB_STATE {
	int ic; // init counter
	int gc; // get counter
	int pc; // put counter
	int dc; // done counter
	FILE *fi; // input file
	FILE *fo; // output file
	int n; // n from input
	int a[__XORLIB_MAX]; // a from input
	int v[__XORLIB_MAX]; // v from solution
	int c[__XORLIB_MAX]; // element counters
} __xorlib_state = {0, 0, 0, 0};

void __xorlib_error(const char *msg) {
	fprintf(stderr, "%s\n", msg);
	exit(EXIT_FAILURE);
}

int init() {
	int i;

	if (__xorlib_state.ic != 0)
		__xorlib_error("Second call to init()");
	++__xorlib_state.ic;

	__xorlib_state.fi = fopen("xor.sis", "rt");
	if (__xorlib_state.fi == 0)
		__xorlib_error("Could not open input file");

	__xorlib_state.fo = fopen("xor.val", "wt");
	if (__xorlib_state.fo == 0)
		__xorlib_error("Could not open output file");

	if (fscanf(__xorlib_state.fi, "%d", &__xorlib_state.n) != 1)
		__xorlib_error("Error reading N from input file");
	if (__xorlib_state.n < 1 || __xorlib_state.n > __XORLIB_MAX)
		__xorlib_error("Invalid N in input file");

	for (i = 0; i < __xorlib_state.n; ++i) {
		if (fscanf(__xorlib_state.fi, "%d", &__xorlib_state.a[i]) != 1)
			__xorlib_error("Error reading A[i] from input file");
		if (__xorlib_state.a[i] < 1 || __xorlib_state.a[i] > __xorlib_state.n)
			__xorlib_error("Invalid A[i] in input file");
		if (__xorlib_state.c[__xorlib_state.a[i] - 1] != 0)
			__xorlib_error("Duplicate A[i] in input file");
		++__xorlib_state.c[__xorlib_state.a[i] - 1];
	}

	fprintf(__xorlib_state.fo, "init() = %d\n", __xorlib_state.n);
	return __xorlib_state.n;
}

int get(int i, int j) {
	int r;

	if (__xorlib_state.ic != 1)
		__xorlib_error("Call to get() before init()");
	if (__xorlib_state.dc != 0)
		__xorlib_error("Call to get() after done()");
	if (__xorlib_state.gc >= __XORLIB_MAX)
		__xorlib_error("Too many calls to get()");
	++__xorlib_state.gc;

	if (i < 1 || i > __xorlib_state.n || j < 1 || j > __xorlib_state.n)
		__xorlib_error("Invalid call to get()");

	r = __xorlib_state.a[i - 1] ^ __xorlib_state.a[j - 1];
	fprintf(__xorlib_state.fo, "get(%d, %d) = %d\n", i, j, r);
	return r;
}

void put(int i, int v) {
	if (__xorlib_state.ic != 1)
		__xorlib_error("Call to put() before init()");
	if (__xorlib_state.dc != 0)
		__xorlib_error("Call to put() after done()");

	if (i < 1 || i > __xorlib_state.n || v < 1 || v > __xorlib_state.n)
		__xorlib_error("Invalid call to put()");
	if (__xorlib_state.c[i - 1] != 1)
		__xorlib_error("Duplicate call to put()");
	++__xorlib_state.c[i - 1];
	++__xorlib_state.pc;

	fprintf(__xorlib_state.fo, "put(%d, %d)\n", i, v);
	__xorlib_state.v[i - 1] = v;
}

void done() {
	int i, w;

	if (__xorlib_state.ic != 1)
		__xorlib_error("Call to done() before init()");
	if (__xorlib_state.dc != 0)
		__xorlib_error("Second call to done()");
	++__xorlib_state.dc;

	if (__xorlib_state.pc != __xorlib_state.n)
		__xorlib_error("Too few calls to put()");

	w = 0;
	for (i = 0; i < __xorlib_state.n; ++i)
		if (__xorlib_state.v[i] != __xorlib_state.a[i])
			++w;

	fprintf(__xorlib_state.fo, "done() -> %s\n", (w == 0 ? "OK" : "Wrong answer"));

	fclose(__xorlib_state.fi);
	fclose(__xorlib_state.fo);
}
