// interface part

function init() : integer; forward;
function get(i, j : integer) : integer; forward;
procedure put(i, v : integer); forward;
procedure done(); forward;

// implementation part
// should be in a unit, but this is not supported by the grading server

const __xorlib_max = 32767;

var __xorlib_state : record
	ic : integer; // init counter
	gc : integer; // get counter
	pc : integer; // put counter
	dc : integer; // done counter
	fi : text; // input file
	fo : text; // output file
	n : integer; // n from input
	a : array[1..__xorlib_max] of integer; // a from input
	v : array[1..__xorlib_max] of integer; // v from solution
	c : array[1..__xorlib_max] of integer; // element counters
end;

procedure __xorlib_error(msg : string);
begin
	writeln(stderr, msg);
	halt(1);
end;

function init() : integer;
var i : integer;
begin
	if __xorlib_state.ic <> 0 then
		__xorlib_error('Second call to init()');
	inc(__xorlib_state.ic);

	assign(__xorlib_state.fi, 'xor.sis'); reset(__xorlib_state.fi);
	if ioresult <> 0 then
		__xorlib_error('Could not open input file');

	assign(__xorlib_state.fo, 'xor.val'); rewrite(__xorlib_state.fo);
	if ioresult <> 0 then
		__xorlib_error('Could not open output file');

	read(__xorlib_state.fi, __xorlib_state.n);
	if ioresult <> 0 then
		__xorlib_error('Error reading N from input file');
	if (__xorlib_state.n < 1) or (__xorlib_state.n > __xorlib_max) then
		__xorlib_error('Invalid N in input file');

	for i := 1 to __xorlib_state.n do begin
		read(__xorlib_state.fi, __xorlib_state.a[i]);
		if ioresult <> 0 then
			__xorlib_error('Error reading A[i] from input file');
		if (__xorlib_state.a[i] < 1) or (__xorlib_state.a[i] > __xorlib_state.n) then
			__xorlib_error('Invalid A[i] in input file');
		if __xorlib_state.c[__xorlib_state.a[i]] <> 0 then
			__xorlib_error('Duplicate A[i] in input file');
		inc(__xorlib_state.c[__xorlib_state.a[i]]);
	end;

	writeln(__xorlib_state.fo, 'init() = ', __xorlib_state.n);
	exit(__xorlib_state.n);
end;

function get(i, j : integer) : integer;
var r : integer;
begin
	if __xorlib_state.ic <> 1 then
		__xorlib_error('Call to get() before init()');
	if __xorlib_state.dc <> 0 then
		__xorlib_error('Call to get() after done()');
	if __xorlib_state.gc >= __xorlib_max then
		__xorlib_error('Too many calls to get()');
	inc(__xorlib_state.gc);

	if (i < 1) or (i > __xorlib_state.n) or (j < 1) or (j > __xorlib_state.n) then
		__xorlib_error('Invalid call to get()');

	r := __xorlib_state.a[i] xor __xorlib_state.a[j];
	writeln(__xorlib_state.fo, 'get(', i, ', ', j, ') = ', r);
	exit(r);
end;

procedure put(i, v : integer);
begin
	if __xorlib_state.ic <> 1 then
		__xorlib_error('Call to put() before init()');
	if __xorlib_state.dc <> 0 then
		__xorlib_error('Call to put() after done()');

	if (i < 1) or (i > __xorlib_state.n) or (v < 1) or (v > __xorlib_state.n) then
		__xorlib_error('Invalid call to put()');
	if __xorlib_state.c[i] <> 1 then
		__xorlib_error('Duplicate call to put()');
	inc(__xorlib_state.c[i]);
	inc(__xorlib_state.pc);

	writeln(__xorlib_state.fo, 'put(', i, ', ', v, ')');
	__xorlib_state.v[i] := v;
end;

procedure done();
var i, w : integer;
begin
	if __xorlib_state.ic <> 1 then
		__xorlib_error('Call to done() before init()');
	if __xorlib_state.dc <> 0 then
		__xorlib_error('Second call to done()');
	inc(__xorlib_state.dc);

	if __xorlib_state.pc <> __xorlib_state.n then
		__xorlib_error('Too few calls to put()');

	w := 0;
	for i := 1 to __xorlib_state.n do
		if __xorlib_state.v[i] <> __xorlib_state.a[i] then
			inc(w);

	if w = 0 then
		writeln(__xorlib_state.fo, 'done() -> OK')
	else
		writeln(__xorlib_state.fo, 'done() -> Wrong answer');

	close(__xorlib_state.fi);
	close(__xorlib_state.fo);
end;
